==> skypager-project-2.51.3.tgz: package/src/helpers/action.js <==
/**
 * REFACTOR NOTES
 *
 * 1) move the definition runner pipeline into this class.
 *     definitions are just designed to provide a configuration DSL, not to determine behavior
 *
 * 2) the purpose of an action is to accept a request. the request should be structured as a
 *    standard flux action.  the action is responsible for processing this action, but will first
 *    validate and record the request before executing it.
 *
 * 3) the action system can handle both mutation requests, and query type requests. it is up to the
 *    person writing the action.  the expectation is that each request will receive a standard type of
 *    response.
 *
 * 4) actions can be exposed over several interfaces:
 *      a) rest
 *      b) cli (commander)
 *      c) ipc
 *      d) streams ? (wip)
*/

import Helper from './helper'
import Registry from '../registry'
import { DSL, ActionDefinition } from './definitions/action'
import { defaults, dotpath, noConflict } from '../util'

import Promise from 'bluebird'

class Action extends Helper {
  static registryInterface (registry) {
    return {
      actionsForCLI: {
        get () {
          return registry.query(action => action.get('definition.interfaces.cli'))
        }
      },
      actionsForIPC: {
        get () {
          return registry.query(action => action.get('definition.interfaces.ipc'))
        }
      },
      endpoints: {
        get () {
          return registry.query(
            action => action.get('definition.interfaces.endpoint')
          )
        }
      }
    }
  }

  /**
   * Configures a CLI utility such as commander to be able to expose
   * this action over a CLI interface.  Any additional context passed
   * from the CLI will be included, and we will make sure the project
   * is set
  */
  attachCLI (commander, dispatchFn) {
    this.project && this.project.warn('using deprecated style of cli actions')

    let builder = this.get('definition.interfaces.cli')

    if (!builder) {
      throw('Can not attach a CLI interface to this action. Missing definition CLI handler')
    }

    if(commander && (typeof(commander.action === 'function') && typeof(commander.command) === 'function') && typeof(commander.option === 'function') ) {
      builder.call(this, commander, dispatchFn)
    }
  }

  /**
   * This creates a promise wrapper around the
   * legacy action definition runner pipeline
  */
  dispatch(...args) {
    let action = this
    let _context = args[ args.length - 1 ]
    let project = this.hostProject

    // getting called through the registry
    if (_context && _context.project && _context.__registryType) {
      project = _context.project
    }

    let context = {
      ...(_context),
      get action() {
        return action
      },
      get currentHelper() {
        return action
      },
      project
    }

    let report = context.report = context.report || new Report(this, {console: false, project, args})

    args.push(context)

    return action.sanityCheck(...args)
      .then(() => action.validateWithGlobalHelpers.call(action, ...args))
      .then(() => action.executeWithGlobalHelpers.call(action, ...args))
      .then((...results) => {
        let context = args[ args.length - 1]
        let { report } = context

        if (!report._done) {
          report.done(...results)
        }

        return report
      })
  }

  /**
   * @private
   *
   * This will get called at the beginning of the action run pipeline
   *
   * It will ensure there is a report, project, and a helpers object
   * on the final context argument.
   */
  sanityCheck(...args) {
    let action = this
    let context = args[ args.length - 1 ]

    if (!context || typeof context !== 'object') {
      throw('Can not call action without the context object')
    }

    if (!context.report && !context.project) {
      throw('Can not call action without report and project context')
    }

    let { project, report } = context

    context.helpers = {
      error: report.error.bind(report),
      warn: report.warn.bind(report),
      info: report.info.bind(report),
      suggest: report.suggest.bind(report),
      done: report.done.bind(report),
      action,
      report,
      project
    }

    return new Promise(function(resolve, reject) {
      if (typeof context === 'object' && typeof(context.project) === 'object' && typeof context.report === 'object') {
         resolve(...args)
      } else {
        console.log(context)
        throw('wtf bro')
      }
    })
  }

  validateWithGlobalHelpers(...args) {
    let action = this
    let context = args[args.length - 1]
    let { project, report, helpers } = context

    let decorated = noConflict(
      action.api.validate.bind(action),
      helpers
    )

    return new Promise(function(resolve, reject) {
      try {
        let pass = decorated(...args)

        if(pass === false || report.errors.length > 0) {
          throw(`Action ${ action.id } failed validation step`)
        } else {
           resolve(...args)
        }
      } catch(error) {
        report.error(error)
        reject(report)
      }
    })
  }

  executeWithGlobalHelpers(...args) {
    let action = this
    let context = args[args.length - 1]
    let { project, report, helpers } = context

    let decorated = noConflict(action.api.execute.bind(project), helpers)

    return new Promise(function(resolve, reject) {
       try {
          let result = decorated(...args)
          resolve(result)
        } catch(error) { reject(error) }
    })
  }

  reportError(error, ...args) {
    this.project && this.project.error('error running action: ' + this.id, error)
    console.error('error running action: ' + this.id, error)
    return false
  }

  get interfaces() {
    return this.definition.interfaces
  }

  get helperClass () {
    return Action
  }

  get definitionClass () {
     return ActionDefinition
  }

  get host() {
     return this.owner && this.owner.host
  }

  get hostProject() {
    return this.project || (
      this.host && this.host.type === 'project' ? this.host : undefined
    )
  }
}

Action.DSL = DSL
Action.Definition = ActionDefinition
Action.apiMethods = ActionDefinition.apiMethods

module.exports = Action

class Report {
  constructor(action, options = {}) {
    this.errors = []
    this.notices = []
    this.warnings = []

    let project = options.project || action.hostProject

    Object.defineProperties(this, {
      options: { enumerable: false, value: options },
      _info: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.info : console.log
          logger(...args)
        }
      },
      _warn: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.warn : console.warn
          logger(...args)
        }
      },
      _error: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.error : console.error
          logger(...args)
        }
      },
      _suggest: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.info : console.log
          logger(...args)
        }
      },
    })
  }

  get success() {
     return this.errors.length === 0
  }

  log(message, ...args) {
    this._info(message, ...args)
  }

  error(message, ...args) {
    this.errors.push(message)
    this._error(message, ...args)
  }

  warn(message, ...args) {
    this.warnings.push(message)
    this._warn(message, ...args)
  }

  suggest(message, ...args) {
    this._info(message, ...args)
  }

  info(message, ...args) {
    this.notices.push(message)
    this._info(message, ...args)
  }

  done(result , ...args) {
    this.result = result
    this._done = true
    this._info('successfully ran action')
    return this
  }
}

==> skypager-project-2.55.2.tgz: package/src/helpers/action.js <==
/**
 * REFACTOR NOTES
 *
 * 1) move the definition runner pipeline into this class.
 *     definitions are just designed to provide a configuration DSL, not to determine behavior
 *
 * 2) the purpose of an action is to accept a request. the request should be structured as a
 *    standard flux action.  the action is responsible for processing this action, but will first
 *    validate and record the request before executing it.
 *
 * 3) the action system can handle both mutation requests, and query type requests. it is up to the
 *    person writing the action.  the expectation is that each request will receive a standard type of
 *    response.
 *
 * 4) actions can be exposed over several interfaces:
 *      a) rest
 *      b) cli (commander)
 *      c) ipc
 *      d) streams ? (wip)
*/

import Helper from './helper'
import Registry from '../registry'
import { DSL, ActionDefinition } from './definitions/action'
import { defaults, dotpath, noConflict } from '../util'

import Promise from 'bluebird'

class Action extends Helper {
  static registryInterface (registry) {
    return {
      actionsForCLI: {
        get () {
          return registry.query(action => action.get('definition.interfaces.cli'))
        }
      },
      actionsForIPC: {
        get () {
          return registry.query(action => action.get('definition.interfaces.ipc'))
        }
      },
      endpoints: {
        get () {
          return registry.query(
            action => action.get('definition.interfaces.endpoint')
          )
        }
      }
    }
  }

  /**
   * Configures a CLI utility such as commander to be able to expose
   * this action over a CLI interface.  Any additional context passed
   * from the CLI will be included, and we will make sure the project
   * is set
  */
  attachCLI (commander, dispatchFn) {
    this.project && this.project.warn('using deprecated style of cli actions')

    let builder = this.get('definition.interfaces.cli')

    if (!builder) {
      throw('Can not attach a CLI interface to this action. Missing definition CLI handler')
    }

    if(commander && (typeof(commander.action === 'function') && typeof(commander.command) === 'function') && typeof(commander.option === 'function') ) {
      builder.call(this, commander, dispatchFn)
    }
  }

  /**
   * This creates a promise wrapper around the
   * legacy action definition runner pipeline
  */
  dispatch(...args) {
    let action = this
    let _context = args[ args.length - 1 ]
    let project = this.hostProject

    // getting called through the registry
    if (_context && _context.project && _context.__registryType) {
      project = _context.project
    }

    let context = {
      ...(_context),
      get action() {
        return action
      },
      get currentHelper() {
        return action
      },
      project
    }

    let report = context.report = context.report || new Report(this, {console: false, project, args})

    args.push(context)

    return action.sanityCheck(...args)
      .then(() => action.validateWithGlobalHelpers.call(action, ...args))
      .then(() => action.executeWithGlobalHelpers.call(action, ...args))
      .then((...results) => {
        let context = args[ args.length - 1]
        let { report } = context

        if (!report._done) {
          report.done(...results)
        }

        return report
      })
  }

  /**
   * @private
   *
   * This will get called at the beginning of the action run pipeline
   *
   * It will ensure there is a report, project, and a helpers object
   * on the final context argument.
   */
  sanityCheck(...args) {
    let action = this
    let context = args[ args.length - 1 ]

    if (!context || typeof context !== 'object') {
      throw('Can not call action without the context object')
    }

    if (!context.report && !context.project) {
      throw('Can not call action without report and project context')
    }

    let { project, report } = context

    context.helpers = {
      error: report.error.bind(report),
      warn: report.warn.bind(report),
      info: report.info.bind(report),
      suggest: report.suggest.bind(report),
      done: report.done.bind(report),
      action,
      report,
      project
    }

    return new Promise(function(resolve, reject) {
      if (typeof context === 'object' && typeof(context.project) === 'object' && typeof context.report === 'object') {
         resolve(...args)
      } else {
        console.log(context)
        throw('wtf bro')
      }
    })
  }

  validateWithGlobalHelpers(...args) {
    let action = this
    let context = args[args.length - 1]
    let { project, report, helpers } = context

    let decorated = noConflict(
      action.api.validate.bind(action),
      helpers
    )

    let failedMessage = `Action ${ action.id } failed validation step`
    return new Promise(function(resolve, reject) {
      try {
        let pass = decorated(...args)
        let passed = Promise.resolve(pass)

        passed.then(result => {
          if(result === false || report.errors.length > 0) {
            report.error(failedMessage)
            reject(report)
          } else {
             resolve(...args)
          }
        }).catch(err => {
          report.error(err)
          report.error(failedMessage)
          reject(report)
        })
      } catch(error) {
        report.error(error)
        report.error(failedMessage)
        reject(report)
      }
    })
  }

  executeWithGlobalHelpers(...args) {
    let action = this
    let context = args[args.length - 1]
    let { project, report, helpers } = context

    let decorated = noConflict(action.api.execute.bind(project), helpers)

    return new Promise(function(resolve, reject) {
       try {
          let result = decorated(...args)
          resolve(result)
        } catch(error) { reject(error) }
    })
  }

  reportError(error, ...args) {
    this.project && this.project.error('error running action: ' + this.id, error)
    console.error('error running action: ' + this.id, error)
    return false
  }

  get interfaces() {
    return this.definition.interfaces
  }

  get helperClass () {
    return Action
  }

  get definitionClass () {
     return ActionDefinition
  }

  get host() {
     return this.owner && this.owner.host
  }

  get hostProject() {
    return this.project || (
      this.host && this.host.type === 'project' ? this.host : undefined
    )
  }
}

Action.DSL = DSL
Action.Definition = ActionDefinition
Action.apiMethods = ActionDefinition.apiMethods

module.exports = Action

class Report {
  constructor(action, options = {}) {
    this.errors = []
    this.notices = []
    this.warnings = []

    let project = options.project || action.hostProject

    Object.defineProperties(this, {
      options: { enumerable: false, value: options },
      _info: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.info : console.log
          logger(...args)
        }
      },
      _warn: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.warn : console.warn
          logger(...args)
        }
      },
      _error: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.error : console.error
          logger(...args)
        }
      },
      _suggest: {
        enumerable: false,
        value: function(...args) {
          let logger = project ? project.info : console.log
          logger(...args)
        }
      },
    })
  }

  get success() {
     return this.errors.length === 0
  }

  log(message, ...args) {
    this._info(message, ...args)
  }

  error(message, ...args) {
    this.errors.push(message)
    this._error(message, ...args)
  }

  warn(message, ...args) {
    this.warnings.push(message)
    this._warn(message, ...args)
  }

  suggest(message, ...args) {
    this._info(message, ...args)
  }

  info(message, ...args) {
    this.notices.push(message)
    this._info(message, ...args)
  }

  done(result , ...args) {
    this.result = result
    this._done = true
    this._info('successfully ran action')
    return this
  }
}
